/***************************************************************************\
  "error.hpp" - Error Handling Functions (Global Phase)
  Revision 17; 10 Nov 2002 - 15 March 2004
  Copyright (C) 2002-2004  Kevin P. Albrecht
\***************************************************************************/

#ifndef ERROR_HPP
#define ERROR_HPP

/////////////////////////
// DEFINES //////////////
/////////////////////////

// External Defines //

// "Program" Error Codes (Controller) //
#define EP_INVALID_ARG                "J101: Invalid command line argument"
#define EP_MISSING_ARG                "J102: Missing command line argument"
#define EP_FILE_OPEN_FAILED           "J103: Error opening file"

// Error Codes for "Front-End" (Phases A, B, and C) //
#define EF_UNDEFINED_TOKEN            "J201: Undefined token"
#define EF_UNDECLARED_IDENTIFIER      "J202: Undeclared identifier"
#define EF_UNEXPECTED_EOF             "J203: Unexpected end-of-file"
#define EF_UNNECESSARY_TYPE_CONV      "J204: Unnecessary type conversion"
#define EF_MISSING_EOL                "J205: Missing end-of-line"
#define EF_MISSING_ASSIGNMENT_OP      "J206: Missing assignment operator"
#define EF_MISSING_RIGHT_PAREN        "J207: Missing right parenthesis"
#define EF_INVALID_EXPRESSION         "J208: Invalid expression"
#define EF_ALREADY_DEFINED            "J209: Identifier already defined"
#define EF_TYPE_INCOMPATIBLE          "J210: Type is incompatible"
#define EF_INVALID_PARSE_POINT        "J211: Invalid parsing point"
#define EF_INCOMPLETE_STR_LIT         "J212: String is never closed"    
#define EF_INCOMPLETE_COMMENT         "J213: Comment is never closed"
#define EF_CONST_ASSIGNMENT           "J214: Cannot assign a new value to a constant"
#define EF_MAX_SCOPES_REACHED         "J215: Maximum concurrent scopes reached"
#define EF_MIN_SCOPES_REACHED         "J216: Minimum scopes reached; Cannot leave global scope"
#define EF_INDENT_ERROR               "J217: Decreased indent must match indent of outer scope"
#define EF_MISSING_TYPE_DEF_OP        "J218: Missing type definition operator"
#define EF_NOT_VALID_TYPE             "J219: Not a valid type"
#define EF_MISSING_EXPRESSION         "J220: Missing expression"
#define EF_ROUT_REDEF                 "J221: Redefinition of routine"
#define EF_NON_MATCHING_RET_TYPE      "J222: Returned type does not match routine"
#define EF_MAIN_NOT_DEF               "J223: 'main' routine not defined"
#define EF_RETURN_NEVER_MADE          "J224: No return from routine"
#define EF_MAIN_NOT_VOID              "J225: 'main' return type should be void, not"
#define EF_NO_RETURN_FOR_VOID_ROUT    "J226: Cannot return a value when routine has no type"
#define EF_EMPTY_BLOCK                "J227: Empty block encountered"
#define EF_BAD_STATEMENT_OUTSIDE_ROUT "J228: Statement illegal outside of routine"
#define EF_CANNOT_COPY_ROUTINE_SYMBOL "J229: Cannot clone a routine"
#define EF_NO_PARAMETER_FOR_ARGUMENT  "J230: Routine call passed too many arguments"
#define EF_ARG_TYPE_MISMATCH          "J231: Argument's type does not match routine definition"
#define EF_ARG_NUM_MISMATCH           "J232: Argument number does not match routine definition"
#define EF_NO_RETURN_VALUE            "J233: Routine does not return a value"
#define EF_RETURN_VAL_NOT_ASSIGNED    "J234: Value returned from routine must be used"
#define EF_MISSING_IDENTIFIER         "J235: Missing identifier"

// Error Codes for "Intermediate Code Manager" (Phase D) //
#define EI_WRITE_AFTER_RESET          "J301: Intermediate code write attempt after reset"
#define EI_READ_BEFORE_RESET          "J302: Intermediate code read attempt before reset"
#define EI_GOTO_BEFORE_RESET          "J303: Intermediate code goto attempt before reset"
#define EI_CURR_INDEX_OUT_OF_BOUNDS   "J304: Current intermediate code index out of bounds"
#define EI_MAX_UNIQUE_NAMES_EXCEEDED  "J305: Maximum number of unique names exceeded"

/////////////////////////
// INCLUDES /////////////
/////////////////////////

// Standard //

// Local //
#include "global.hpp"
#include "k_string.hpp"

/////////////////////////
// DECLARATIONS /////////
/////////////////////////

void err (bool lines, char *error, char *extra);
void errm (char *s1, char *s2);
void errterm ();

/////////////////////////
// DEFINITIONS //////////
/////////////////////////

//// err
//// Display an error
void err (bool lines, char *error, char *extra)
{
  Global::nErrors++;

  if (lines)
    cerr << Global::sCurrentFilename << "(" << Global::nCurrentLine << ") : ";

  cerr << "Error " << error;

  if (extra)
    cerr << ": " << extra;

  cerr << "\n";
}

//// errm
//// Display an additional error message
void errm (char *s1, char *s2)
{
  cerr << "      (more) ";

  if (s1)
    cerr << s1;

  if (s2)
    cerr << ": " << s2;

  cerr << "\n";
}

//// errterm
//// Terminate the program due to an error
void errterm ()
{
  cerr << "Fatal Error: Terminating program...\n";
  exit (0);
}

#endif
