/***************************************************************************\
  "token.hpp" - Token Variables and Functions (Phase B)
  Revision 20; 10 Nov 2002 - 15 March 2004
  Copyright (C) 2002-2004  Kevin P. Albrecht
\***************************************************************************/

#ifndef TOKEN_HPP
#define TOKEN_HPP

/////////////////////////
// DEFINES //////////////
/////////////////////////

// External Defines //
// GLOBAL_TOKEN_SIZE

/////////////////////////
// INCLUDES /////////////
/////////////////////////

// Standard //

// Local //

/////////////////////////
// DECLARATIONS /////////
/////////////////////////

typedef enum
{
  // null //
  codeNull, codeError,

  // codeStart is passed to lex_get at the start of the file //
  codeStart,

  // valued token types //
  codeIdentifier, codeBoolLiteral, codeIntLiteral,
  codeRatLiteral, codeStrLiteral,

  // source code whitespace types //
  codeEOF, codeEOL,

  // reserved words //
  codeOR,
  codeAND, codeDEF, codeINT, codeRAT, codeSTR, codePOP, codeMOD,
  codeFILE, codeGOTO, codeCOND, codeLOOP, codePUSH, codeROUT,
  codeBOOL,
  codeCONST, 
  codeRETURN,
  code_PRIM_CIN, code_PRIM_ASC, code_PRIM_COUT,
  code_PRIM_RAND, code_PRIM_STOP,

  // file i/o primitives
  code_PRIM_FIO_OR, code_PRIM_FIO_OW, code_PRIM_FIO_C,
  code_PRIM_FIO_R, code_PRIM_FIO_W, code_PRIM_FIO_EOF,

  // specials //
  codePLUS, codeMINUS, codeTIMES, codeFORWARDSLASH,
  codeBACKSLASH, codeEQUALS, codeLEFTPAREN, codeRIGHTPAREN,
  codeCOMMA, codeLT, codeGT, codeDOLLAR,
  codeDOT, codeLEFTBRACK, codeRIGHTBRACK, codeQUOTEMARK,
  codeSEMICOLON, codeCOLON, codeNOTEQUAL, codeLTEQUAL,
  codeGTEQUAL, codeLEFTBRACE, codeRIGHTBRACE, codeANDSIGN,
  codePOUNDSIGN, codeARROW, codeEXPONENT
} TokenCode;

char *gvsTokenCodeStrings[] =
{
  // null //
  "<null>", "<error>",

  // codeStart is passed to lex_get at the start of the file //    
  "<start>",

  // valued token types //
  "<identifier>", "<bool_literal>", "<int_literal>",
  "<rat_literal>", "<str_literal>", 

  // source code whitespace types //
  "<eof>", "<eol>",

  // reserved words //
  "or",
  "and", "def", "int", "rat", "str", "pop", "mod",
  "file", "goto", "cond", "loop", "push", "rout",
  "bool",
  "const",
  "return",
  "_prim_cin", "_prim_asc", "_prim_cout",
  "_prim_rand", "_prim_stop",

  // file i/o primitives
  "_prim_fio_or", "_prim_fio_ow", "_prim_fio_c",
  "_prim_fio_r", "_prim_fio_w", "_prim_fio_eof",

  // specials //
  "+", "-", "*", "/",
  "\\", "=", "(", ")",
  ",", "<", ">", "$",
  ".", "[", "]", "\"",
  ";", ":", "<>", "<=",
  ">=", "{", "}", "&",
  "#", "<-", "^"
};

class Token
{
  public:
    Token ();
    ~Token ();

    void set (void *aValue);
    void clear ();

    TokenCode code;
    char lexeme[GLOBAL_TOKEN_SIZE]; // the actual text of the token as it appears in input
    void *value; // the value held in the token:
                 // CODE            VALUE          MEANING
                 // ----            -----          -------
                 // codeIntLiteral  int*           integer
                 // codeRatLiteral  double*        rational
                 // codeStrLiteral  char*          string
                 // (other)         void*          (none)
};

/////////////////////////
// DEFINITIONS //////////
/////////////////////////

Token::Token ()
{
  clear ();
}

Token::~Token ()
{
  if (code == codeStrLiteral)
  {
    // this is a bug fix so that deleting an empty string won't cause an error.
    if (((char*)(value))[0] != 0)
      delete value;
  }
  else
  {
    delete value;
  }
}

void Token::clear ()
{
  code = codeNull;
  k_copy (lexeme, "");
  value = NULL;
}

void Token::set (void *aValue)
{
  value = aValue;
}

#endif
